package com.zhilian.online.config;

import com.zhilian.common.redis.service.RedisService;
import com.zhilian.online.uitls.MessageResolver;
import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.producer.KafkaProducer;
import org.apache.kafka.clients.producer.ProducerConfig;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.apache.kafka.common.serialization.StringSerializer;
import org.eclipse.paho.client.mqttv3.*;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.Properties;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * @BelongsProject: smart-cloud-server
 * @BelongsPackage: com.zhilian.online.config
 * @Author: LiYuan
 * @CreateTime: 2024-03-31  09:45
 * @Description: TODO
 * @Version: 1.0
 */
@Component
@Slf4j
public class MqttxConfig {

    /**
     * 路径
     */
    @Value("${mqtt.server.broker}")
    private String broker;

    /**
     * 主题
     */
    @Value("${mqtt.server.topic}")
    private String topic;

    /**
     * 客户端ID
     */
    @Value("${mqtt.server.clientId}")
    private String clientId;

    /**
     * 用户名
     */
    @Value("${mqtt.server.username}")
    private String username;

    /**
     * 密码
     */
    @Value("${mqtt.server.password}")
    private String password;

    /**
     * qos
     */
    @Value("${mqtt.server.qos}")
    private Integer qos;

    /**
     * redis服务
     */
    @Autowired
    private RedisService redisService;
    /**
     * kafka订阅topic
     */
    private String topicName = "test-topic";

    /**
     * kafka服务
     */
    @Value("${spring.kafka.producer.bootstrap-servers}")
    private String bootstrap_servers;

    /**
     * 线程池
     */
    private ExecutorService executorService = Executors.newFixedThreadPool(3);

    /**
     * kafka服务
     */
    private KafkaProducer<String, String> kafkaProducer;

    /**
     * @return
     * @Description: 初始化kafka服务
     */
    @PostConstruct
    public KafkaProducer<Object, Object> initKafkaProducer() {
        Properties properties = new Properties();
        properties.put(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, bootstrap_servers);
        properties.put(ProducerConfig.KEY_SERIALIZER_CLASS_CONFIG, StringSerializer.class.getName());
        properties.put(ProducerConfig.VALUE_SERIALIZER_CLASS_CONFIG, StringSerializer.class.getName());

        return new KafkaProducer<>(properties);
    }

    /**
     * @param msg
     * @Description: 发送消息
     */
    private void sendMsg(String msg) {
        kafkaProducer.send(new ProducerRecord<>(topicName, msg));
        log.info("Kafka在topic:{}中发送消息{}", topicName, msg);
    }

    @PostConstruct
    public void initMqtt() {
        log.info("mqttx连接中......");

        try {
            MqttClient mqttClient = new MqttClient(broker, clientId, new MemoryPersistence());

            MqttConnectOptions mqttConnectOptions = new MqttConnectOptions();

            mqttConnectOptions.setUserName(username);
            mqttConnectOptions.setPassword(password.toCharArray());

            //链接超时
            mqttConnectOptions.setConnectionTimeout(60);
            //心跳检测
            mqttConnectOptions.setKeepAliveInterval(60);
            mqttClient.connect(mqttConnectOptions);
            log.info("mqtt{}服务连接成功", broker);

            //回调
            mqttClient.setCallback(new MqttCallback() {
                @Override
                public void connectionLost(Throwable throwable) {
                    log.error("连接断开{}", throwable.getMessage());
                    //链接断开后会尝试重新连接
                    while (mqttClient.isConnected()) {
                        try {
                            //每次重连间隔60秒
                            Thread.sleep(1000 * 60);
                            mqttClient.connect();
                        } catch (Exception e) {
                            throw new RuntimeException(e);
                        }
                    }
                }

                @Override
                public void messageArrived(String s, MqttMessage mqttMessage) throws Exception {
                    log.info("消息到达,接受消息主题{},接受消息Qos{},接受消息内容{}", topic, mqttMessage.getQos(), new String(mqttMessage.getPayload()));

                    //将接受到的车辆报文存储到kafka中
                    executorService.execute(() -> {
                        //解析得出原始报文String
                        String sourceMsg = new String(mqttMessage.getPayload());
                        //对原始报文进行解析
                        String parseMsg = MessageResolver.parseMsg(sourceMsg);

                        //截取原始报文获得车辆VIN码
                        String vin = parseMsg.substring(0, 17);

                        log.info("当前车辆VIN码:{}",vin);

                        //调取接口,查询是否是我们的车



                    });

                }

                @Override
                public void deliveryComplete(IMqttDeliveryToken iMqttDeliveryToken) {
                    log.info("消息{}发送成功" + iMqttDeliveryToken.isComplete());
                }
            });

            mqttClient.subscribe(topic, qos);

        } catch (MqttException e) {
            throw new RuntimeException(e);
        }


    }


}
